@@ -46,12 +46,11 @@
 #include <unistd.h>
 #include <getopt.h>
 #include <efivar.h>
+#include <efiboot.h>
 #include <inttypes.h>
 #include "list.h"
 #include "efi.h"
-#include "efichar.h"
 #include "unparse_path.h"
-#include "disk.h"
 #include "efibootmgr.h"
 
 
@@ -68,7 +67,7 @@ typedef struct _var_entry {
 	uint32_t	attributes;
 	uint16_t	num;
 	list_t		list;
-} efi_variable_t;
+} var_entry_t;
 
 /* global variables */
 static	LIST_HEAD(boot_entry_list);
@@ -79,10 +78,10 @@ static void
 free_vars(list_t *head)
 {
 	list_t *pos, *n;
-	efi_variable_t *boot;
+	var_entry_t *boot;
 
 	list_for_each_safe(pos, n, head) {
-		boot = list_entry(pos, efi_variable_t, list);
+		boot = list_entry(pos, var_entry_t, list);
 		if (boot->data)
 			free(boot->data);
 		list_del(&(boot->list));
@@ -94,7 +93,7 @@ static void
 read_vars(char **namelist,
 	  list_t *head)
 {
-	efi_variable_t *entry;
+	var_entry_t *entry;
 	int i, rc;
 
 	if (!namelist)
@@ -102,16 +101,20 @@ read_vars(char **namelist,
 
 	for (i=0; namelist[i] != NULL; i++) {
 		if (namelist[i]) {
-			entry = malloc(sizeof(efi_variable_t));
+			entry = malloc(sizeof(var_entry_t));
 			if (!entry)
 				goto err;
-			memset(entry, 0, sizeof(efi_variable_t));
+			memset(entry, 0, sizeof(var_entry_t));
 
 			rc = efi_get_variable(EFI_GLOBAL_GUID, namelist[i],
 					       &entry->data, &entry->data_size,
 					       &entry->attributes);
-			if (rc < 0)
-				goto err;
+			if (rc < 0) {
+				warn("Skipping unreadable variable \"%s\"",
+					namelist[i]);
+				free(entry);
+				continue;
+			}
 
 			/* latest apple firmware sets high bit which appears
 			 * invalid to the linux kernel if we write it back so
@@ -167,16 +170,20 @@ find_free_boot_var(list_t *boot_list)
 	int num_vars=0, i=0, found;
 	uint16_t *vars, free_number;
 	list_t *pos;
-	efi_variable_t *boot;
-	list_for_each(pos, boot_list) {
+	var_entry_t *boot;
+
+	list_for_each(pos, boot_list)
 		num_vars++;
-	}
+
+	if (num_vars == 0)
+		return 0;
+
 	vars = calloc(1, sizeof(uint16_t) * num_vars);
 	if (!vars)
 		return -1;
 
 	list_for_each(pos, boot_list) {
-		boot = list_entry(pos, efi_variable_t, list);
+		boot = list_entry(pos, var_entry_t, list);
 		vars[i] = boot->num;
 		i++;
 	}
@@ -187,7 +194,7 @@ find_free_boot_var(list_t *boot_list)
 	for (free_number = 0; free_number < num_vars && found; free_number++) {
 		found = 0;
 		list_for_each(pos, boot_list) {
-			boot = list_entry(pos, efi_variable_t, list);
+			boot = list_entry(pos, var_entry_t, list);
 			if (boot->num == free_number) {
 				found = 1;
 				break;
@@ -206,15 +213,16 @@ static void
 warn_duplicate_name(list_t *boot_list)
 {
 	list_t *pos;
-	efi_variable_t *boot;
-	EFI_LOAD_OPTION *load_option;
+	var_entry_t *boot;
+	efi_load_option *load_option;
+	const unsigned char const *desc;
 
 	list_for_each(pos, boot_list) {
-		boot = list_entry(pos, efi_variable_t, list);
-		load_option = (EFI_LOAD_OPTION *)
+		boot = list_entry(pos, var_entry_t, list);
+		load_option = (efi_load_option *)
 			boot->data;
-		if (!efichar_char_strcmp(opts.label,
-					 load_option->description)) {
+		desc = efi_loadopt_desc(load_option);
+		if (!strcmp((char *)opts.label, (char *)desc)) {
 			fprintf(stderr, "** Warning ** : %.8s has same label %s\n",
 			       boot->name,
 			       opts.label);
@@ -222,19 +230,22 @@ warn_duplicate_name(list_t *boot_list)
 	}
 }
 
-static efi_variable_t *
+static var_entry_t *
 make_boot_var(list_t *boot_list)
 {
-	efi_variable_t *boot = NULL;
+	var_entry_t *boot = NULL;
 	int free_number;
 	list_t *pos;
 	int rc;
+	uint8_t *extra_args = NULL;
+	ssize_t extra_args_size = 0;
+	ssize_t needed=0, sz;
 
-	if (opts.bootnum == -1)
+	if (opts.bootnum == -1) {
 		free_number = find_free_boot_var(boot_list);
-	else {
+	} else {
 		list_for_each(pos, boot_list) {
-			boot = list_entry(pos, efi_variable_t, list);
+			boot = list_entry(pos, var_entry_t, list);
 			if (boot->num == opts.bootnum) {
 				fprintf(stderr, "** Warning ** : bootnum %04X "
 				        "already exists\n", opts.bootnum);
@@ -249,26 +260,46 @@ make_boot_var(list_t *boot_list)
 		return NULL;
 	}
 
-	/* Create a new efi_variable_t object
+	/* Create a new var_entry_t object
 	   and populate it.
 	*/
-
 	boot = calloc(1, sizeof(*boot));
 	if (!boot) {
 		warn("efibootmgr");
 		return NULL;
 	}
-	if (make_linux_load_option(&boot->data, &boot->data_size) < 0)
-		goto err_boot_entry;
-	if (append_extra_args(&boot->data, &boot->data_size) < 0)
-		goto err_boot_entry;
+
+	sz = get_extra_args(NULL, 0);
+	if (sz < 0)
+		goto err;
+	extra_args_size = sz;
+
+	boot->data = NULL;
+	boot->data_size = 0;
+	needed = make_linux_load_option(&boot->data, &boot->data_size,
+					NULL, sz);
+	if (needed < 0)
+		goto err;
+	boot->data_size = needed;
+	boot->data = malloc(needed);
+	if (!boot->data)
+		goto err;
+
+	extra_args = boot->data + needed - extra_args_size;
+	sz = get_extra_args(extra_args, extra_args_size);
+	if (sz < 0)
+		goto err;
+	sz = make_linux_load_option(&boot->data, &boot->data_size,
+				    extra_args, extra_args_size);
+	if (sz < 0)
+		goto err;
 
 	boot->num = free_number;
-	boot->guid = EFI_GLOBAL_VARIABLE;
+	boot->guid = EFI_GLOBAL_GUID;
 	rc = asprintf(&boot->name, "Boot%04X", free_number);
 	if (rc < 0) {
 		warn("efibootmgr");
-		goto err_boot_entry;
+		goto err;
 	}
 	boot->attributes = EFI_VARIABLE_NON_VOLATILE |
 			    EFI_VARIABLE_BOOTSERVICE_ACCESS |
@@ -276,27 +307,29 @@ make_boot_var(list_t *boot_list)
 	rc = efi_set_variable(boot->guid, boot->name, boot->data,
 				boot->data_size, boot->attributes);
 	if (rc < 0)
-		goto err_boot_entry;
+		goto err;
 	list_add_tail(&boot->list, boot_list);
 	return boot;
-err_boot_entry:
-	if (boot->name) {
-		warn("Could not set variable %s", boot->name);
-		free(boot->name);
-	} else {
-		warn("Could not set variable");
+err:
+	if (boot) {
+		if (boot->data)
+			free(boot->data);
+		if (boot->name) {
+			warn("Could not set variable %s", boot->name);
+			free(boot->name);
+		} else {
+			warn("Could not set variable");
+		}
+		free(boot);
 	}
-	if (boot->data)
-		free(boot->data);
-	free(boot);
 	return NULL;
 }
 
 static int
-read_boot_order(efi_variable_t **boot_order)
+read_boot_order(var_entry_t **boot_order)
 {
 	int rc;
-	efi_variable_t *new = NULL, *bo = NULL;
+	var_entry_t *new = NULL, *bo = NULL;
 
 	if (*boot_order == NULL) {
 		new = calloc(1, sizeof (**boot_order));
@@ -336,7 +369,7 @@ set_boot_u16(const char *name, uint16_t num)
 static int
 add_to_boot_order(uint16_t num)
 {
-	efi_variable_t *boot_order = NULL;
+	var_entry_t *boot_order = NULL;
 	uint64_t new_data_size;
 	uint16_t *new_data, *old_data;
 	int rc;
@@ -375,7 +408,7 @@ add_to_boot_order(uint16_t num)
 static int
 remove_dupes_from_boot_order(void)
 {
-	efi_variable_t *boot_order = NULL;
+	var_entry_t *boot_order = NULL;
 	uint64_t new_data_size;
 	uint16_t *new_data, *old_data;
 	unsigned int old_i,new_i;
@@ -429,9 +462,8 @@ remove_dupes_from_boot_order(void)
 static int
 remove_from_boot_order(uint16_t num)
 {
-	efi_variable_t *boot_order = NULL;
-	uint64_t new_data_size;
-	uint16_t *new_data, *old_data;
+	var_entry_t *boot_order = NULL;
+	uint16_t *data;
 	unsigned int old_i,new_i;
 	int rc;
 
@@ -443,34 +475,37 @@ remove_from_boot_order(uint16_t num)
 	}
 
 	/* We've now got an array (in boot_order->data) of the
-	   boot order.  Simply copy the array, skipping the
-	   entry we're deleting.
+	   boot order. Squeeze out any instance of the entry we're
+	   deleting by shifting the remainder down.
 	*/
-	old_data = (uint16_t *)(boot_order->data);
-	/* Start with the same size */
-	new_data_size = boot_order->data_size - sizeof (*new_data);
-	new_data = malloc(new_data_size);
-	if (!new_data)
-		return -1;
+	data = (uint16_t *)(boot_order->data);
 
 	for (old_i=0,new_i=0;
-	     old_i < boot_order->data_size / sizeof(*new_data);
+	     old_i < boot_order->data_size / sizeof(data[0]);
 	     old_i++) {
-		if (old_data[old_i] != num) {
-				/* Copy this value */
-			new_data[new_i] = old_data[old_i];
+		if (data[old_i] != num) {
+			if (new_i != old_i)
+				data[new_i] = data[old_i];
 			new_i++;
 		}
 	}
 
-	/* Now new_data has what we need */
-	free(boot_order->data);
-	boot_order->data = (uint8_t *)new_data;
-	boot_order->data_size = new_data_size;
+	/* If nothing removed, no need to update the BootOrder variable */
+	if (new_i == old_i)
+		goto all_done;
+
+	/* BootOrder variable needs to be updated */
 	efi_del_variable(EFI_GLOBAL_GUID, "BootOrder");
+
+	if (new_i == 0)
+		goto all_done;
+
+	boot_order->data_size = sizeof(data[0]) * new_i;
 	rc = efi_set_variable(EFI_GLOBAL_GUID, "BootOrder", boot_order->data,
 				boot_order->data_size, boot_order->attributes);
+all_done:
 	free(boot_order->data);
+	free(boot_order);
 	return rc;
 }
 
@@ -505,7 +540,7 @@ delete_boot_var(uint16_t num)
 	int rc;
 	char name[16];
 	list_t *pos, *n;
-	efi_variable_t *boot;
+	var_entry_t *boot;
 
 	snprintf(name, sizeof(name), "Boot%04X", num);
 	rc = efi_del_variable(EFI_GLOBAL_GUID, name);
@@ -526,7 +561,7 @@ delete_boot_var(uint16_t num)
 		return rc;
 	}
 	list_for_each_safe(pos, n, &boot_entry_list) {
-		boot = list_entry(pos, efi_variable_t, list);
+		boot = list_entry(pos, var_entry_t, list);
 		if (boot->num == num) {
 			rc = remove_from_boot_order(num);
 			if (rc < 0)
@@ -542,13 +577,13 @@ static void
 set_var_nums(list_t *list)
 {
 	list_t *pos;
-	efi_variable_t *var;
+	var_entry_t *var;
 	int num=0, rc;
 	char *name;
 	int warn=0;
 
 	list_for_each(pos, list) {
-		var = list_entry(pos, efi_variable_t, list);
+		var = list_entry(pos, var_entry_t, list);
 		rc = sscanf(var->name, "Boot%04X-%*s", &num);
 		if (rc == 1) {
 			var->num = num;
@@ -569,7 +604,7 @@ set_var_nums(list_t *list)
 }
 
 static void
-unparse_boot_order(uint16_t *order, int length)
+print_boot_order(uint16_t *order, int length)
 {
 	int i;
 	printf("BootOrder: ");
@@ -585,10 +620,10 @@ static int
 is_current_boot_entry(int b)
 {
 	list_t *pos;
-	efi_variable_t *boot;
+	var_entry_t *boot;
 
 	list_for_each(pos, &boot_entry_list) {
-		boot = list_entry(pos, efi_variable_t, list);
+		boot = list_entry(pos, var_entry_t, list);
 		if (boot->num == b)
 			return 1;
 	}
@@ -684,7 +719,7 @@ static int
 construct_boot_order(char *bootorder, int keep,
 			uint16_t **ret_data, size_t *ret_data_size)
 {
-	efi_variable_t bo;
+	var_entry_t bo;
 	int rc;
 	uint16_t *data = NULL;
 	size_t data_size = 0;
@@ -764,100 +799,85 @@ set_boot_order(int keep_old_entries)
 	if (rc < 0 || data_size == 0)
 		return rc;
 
-	return efi_set_variable(EFI_GLOBAL_GUID, "BootOrder", data, data_size,
-				EFI_VARIABLE_NON_VOLATILE |
-				EFI_VARIABLE_BOOTSERVICE_ACCESS |
-				EFI_VARIABLE_RUNTIME_ACCESS);
+	rc = efi_set_variable(EFI_GLOBAL_GUID, "BootOrder", data, data_size,
+			      EFI_VARIABLE_NON_VOLATILE |
+			      EFI_VARIABLE_BOOTSERVICE_ACCESS |
+			      EFI_VARIABLE_RUNTIME_ACCESS);
+	free(data);
+	return rc;
 }
 
 static void
 show_boot_vars()
 {
 	list_t *pos;
-	efi_variable_t *boot;
-	char description[80];
-	EFI_LOAD_OPTION *load_option;
-	EFI_DEVICE_PATH *path;
-	unsigned long optional_data_len=0;
+	var_entry_t *boot;
+	const unsigned char const *description;
+	efi_load_option *load_option;
+	efidp dp = NULL;
+	unsigned char *optional_data = NULL;
+	size_t optional_data_len=0;
 
 	list_for_each(pos, &boot_entry_list) {
-		boot = list_entry(pos, efi_variable_t, list);
-		load_option = (EFI_LOAD_OPTION *)boot->data;
-		efichar_to_char(description,
-				load_option->description, sizeof(description));
-		path = load_option_path(load_option);
+		boot = list_entry(pos, var_entry_t, list);
+		load_option = (efi_load_option *)boot->data;
+		description = efi_loadopt_desc(load_option);
+		dp = efi_loadopt_path(load_option);
 		if (boot->name)
 			printf("%.8s", boot->name);
 		else
 			printf("Boot%04X", boot->num);
 
-		if (load_option->attributes & LOAD_OPTION_ACTIVE)
-			printf("* ");
-		else    printf("  ");
+		printf("%c ", (efi_loadopt_attrs(load_option)
+			       & LOAD_OPTION_ACTIVE) ? '*' : ' ');
 		printf("%s", description);
 
 		if (opts.verbose) {
 			char *text_path = NULL;
 			size_t text_path_len = 0;
+			uint16_t pathlen = efi_loadopt_pathlen(load_option);
 			ssize_t rc;
 
-			rc = unparse_path(text_path, text_path_len, path,
-				     load_option->file_path_list_length);
-			if (rc < 0) {
-				fprintf(stderr, "Could not parse device path: %m\n");
-				exit(1);
-			}
+			rc = efidp_format_device_path(text_path, text_path_len,
+						      dp, pathlen);
+			if (rc < 0)
+				err(18, "Could not parse device path");
 			rc += 1;
 
 			text_path_len = rc;
 			text_path = calloc(1, rc);
-			if (!text_path) {
-				fprintf(stderr, "Could not parse device path: %m\n");
-				exit(1);
-			}
-			rc = unparse_path(text_path, text_path_len,
-				path, load_option->file_path_list_length);
-			if (rc < 0) {
-				fprintf(stderr, "Could not parse device path: %m\n");
-				exit(1);
-			}
+			if (!text_path)
+				err(19, "Could not parse device path");
+
+			rc = efidp_format_device_path(text_path, text_path_len,
+						      dp, pathlen);
+			if (rc < 0)
+				err(20, "Could not parse device path");
 			printf("\t%s", text_path);
 			free(text_path);
 			text_path_len = 0;
 			/* Print optional data */
 
-			optional_data_len =
-				boot->data_size -
-				load_option->file_path_list_length -
-				((char *)path - (char *)load_option);
-			if (optional_data_len <= 0) {
-				printf("\n");
-				continue;
-			}
+			rc = efi_loadopt_optional_data(load_option,
+							   boot->data_size,
+							   &optional_data,
+							   &optional_data_len);
+			if (rc < 0)
+				err(21, "Could not parse optional data");
 
-			rc = unparse_raw_text(NULL, 0,
-				((uint8_t *)path)
-					+ load_option->file_path_list_length,
-				optional_data_len);
-			if (rc < 0) {
-				fprintf(stderr, "Could not parse device path: %m\n");
-				exit(1);
-			}
+			rc = unparse_raw_text(NULL, 0, optional_data,
+					      optional_data_len);
+			if (rc < 0)
+				err(22, "Could not parse optional data");
 			rc += 1;
 			text_path_len = rc;
 			text_path = calloc(1, rc);
-			if (!text_path) {
-				fprintf(stderr, "Could not parse device path: %m\n");
-				exit(1);
-			}
+			if (!text_path)
+				err(23, "Could not parse optional data");
 			rc = unparse_raw_text(text_path, text_path_len,
-				((uint8_t *)path)
-					+ load_option->file_path_list_length,
-				optional_data_len);
-			if (rc < 0) {
-				fprintf(stderr, "Could not parse device path: %m\n");
-				exit(1);
-			}
+					      optional_data, optional_data_len);
+			if (rc < 0)
+				err(23, "Could not parse device path");
 			printf("%s", text_path);
 			free(text_path);
 		}
@@ -869,22 +889,17 @@ static void
 show_boot_order()
 {
 	int rc;
-	efi_variable_t *boot_order = NULL;
+	var_entry_t *boot_order = NULL;
 	uint16_t *data;
 
 	rc = read_boot_order(&boot_order);
 
 	if (rc < 0) {
-		if (errno == ENOENT) {
-			boot_order = calloc(1, sizeof (*boot_order));
-			if (!boot_order) {
-				perror("show_boot_order()");
-				return;
-			}
-		} else {
+		if (errno == ENOENT)
+			printf("No BootOrder is set; firmware will attempt recovery\n");
+		else
 			perror("show_boot_order()");
-			return;
-		}
+		return;
 	}
 
 	/* We've now got an array (in boot_order->data) of the
@@ -892,7 +907,7 @@ show_boot_order()
 	*/
 	data = (uint16_t *)boot_order->data;
 	if (boot_order->data_size) {
-		unparse_boot_order(data, boot_order->data_size / sizeof(uint16_t));
+		print_boot_order(data, boot_order->data_size / sizeof(uint16_t));
 		free(boot_order->data);
 	}
 	free(boot_order);
@@ -902,20 +917,20 @@ static int
 set_active_state()
 {
 	list_t *pos;
-	efi_variable_t *boot;
-	EFI_LOAD_OPTION *load_option;
+	var_entry_t *boot;
+	efi_load_option *load_option;
 
 	list_for_each(pos, &boot_entry_list) {
-		boot = list_entry(pos, efi_variable_t, list);
-		load_option = (EFI_LOAD_OPTION *)boot->data;
+		boot = list_entry(pos, var_entry_t, list);
+		load_option = (efi_load_option *)boot->data;
 		if (boot->num == opts.bootnum) {
 			if (opts.active == 1) {
-				if (load_option->attributes
+				if (efi_loadopt_attrs(load_option)
 						& LOAD_OPTION_ACTIVE) {
 					return 0;
 				} else {
-					load_option->attributes
-						|= LOAD_OPTION_ACTIVE;
+					efi_loadopt_attr_set(load_option,
+							LOAD_OPTION_ACTIVE);
 					return efi_set_variable(boot->guid,
 							boot->name,
 							boot->data,
@@ -924,12 +939,12 @@ set_active_state()
 				}
 			}
 			else if (opts.active == 0) {
-				if (!(load_option->attributes
+				if (!(efi_loadopt_attrs(load_option)
 						& LOAD_OPTION_ACTIVE)) {
 					return 0;
 				} else {
-					load_option->attributes
-						&= ~LOAD_OPTION_ACTIVE;
+					efi_loadopt_attr_clear(load_option,
+							LOAD_OPTION_ACTIVE);
 					return efi_set_variable(boot->guid,
 							boot->name,
 							boot->data,
@@ -945,9 +960,6 @@ set_active_state()
 	return -1;
 }
 
-
-
-
 static void
 usage()
 {
@@ -958,13 +970,21 @@ usage()
 	printf("\t-b | --bootnum XXXX   modify BootXXXX (hex)\n");
 	printf("\t-B | --delete-bootnum delete bootnum (hex)\n");
 	printf("\t-c | --create         create new variable bootnum and add to bootorder\n");
+	printf("\t-C | --create-only	create new variable bootnum and do not add to bootorder\n");
 	printf("\t-D | --remove-dups	remove duplicate values from BootOrder\n");
 	printf("\t-d | --disk disk       (defaults to /dev/sda) containing loader\n");
 	printf("\t-e | --edd [1|3|-1]   force EDD 1.0 or 3.0 creation variables, or guess\n");
 	printf("\t-E | --device num      EDD 1.0 device number (defaults to 0x80)\n");
 	printf("\t-g | --gpt            force disk with invalid PMBR to be treated as GPT\n");
-	printf("\t-H | --acpi_hid XXXX  set the ACPI HID (used with -i)\n");
 	printf("\t-i | --iface name     create a netboot entry for the named interface\n");
+#if 0
+	printf("\t     --ip-addr <local>,<remote>	set local and remote IP addresses\n");
+	printf("\t     --ip-gateway <gateway>	set the network gateway\n");
+	printf("\t     --ip-netmask <netmask>	set the netmask or prefix length\n");
+	printf("\t     --ip-proto TCP|UDP	set the IP protocol to be used\n");
+	printf("\t     --ip-port <local>,<remote>	set local and remote IP ports\n");
+	printf("\t     --ip-origin { {dhcp|static} | { static|stateless|stateful} }\n");
+#endif
 	printf("\t-l | --loader name     (defaults to \\EFI\\redhat\\grub.efi)\n");
 	printf("\t-L | --label label     Boot manager display label (defaults to \"Linux\")\n");
 	printf("\t-n | --bootnext XXXX   set BootNext to XXXX (hex)\n");
@@ -976,7 +996,6 @@ usage()
 	printf("\t-t | --timeout seconds  set boot manager timeout waiting for user input.\n");
 	printf("\t-T | --delete-timeout   delete Timeout.\n");
 	printf("\t-u | --unicode | --UCS-2  pass extra args as UCS-2 (default is ASCII)\n");
-	printf("\t-U | --acpi_uid XXXX    set the ACPI UID (used with -i)\n");
 	printf("\t-v | --verbose          print additional information\n");
 	printf("\t-V | --version          return version and exit\n");
 	printf("\t-w | --write-signature  write unique sig to MBR if needed\n");
@@ -994,12 +1013,9 @@ set_default_opts()
 	opts.timeout         = -1;   /* Don't set it */
 	opts.edd10_devicenum = 0x80;
 	opts.loader          = "\\EFI\\redhat\\grub.efi";
-	opts.label           = "Linux";
+	opts.label           = (unsigned char *)"Linux";
 	opts.disk            = "/dev/sda";
-	opts.iface           = NULL;
 	opts.part            = 1;
-	opts.acpi_hid        = -1;
-	opts.acpi_uid        = -1;
 }
 
 static void
@@ -1019,10 +1035,10 @@ parse_opts(int argc, char **argv)
 			{"bootnum",          required_argument, 0, 'b'},
 			{"delete-bootnum",         no_argument, 0, 'B'},
 			{"create",                 no_argument, 0, 'c'},
+			{"create-only",		   no_argument, 0, 'C'},
 			{"remove-dups",            no_argument, 0, 'D'},
 			{"disk",             required_argument, 0, 'd'},
 			{"iface",            required_argument, 0, 'i'},
-			{"acpi_hid",         required_argument, 0, 'H'},
 			{"edd-device",       required_argument, 0, 'E'},
 			{"edd30",            required_argument, 0, 'e'},
 			{"gpt",                    no_argument, 0, 'g'},
@@ -1039,7 +1055,6 @@ parse_opts(int argc, char **argv)
 			{"delete-timeout",         no_argument, 0, 'T'},
 			{"unicode",                no_argument, 0, 'u'},
 			{"UCS-2",                  no_argument, 0, 'u'},
-			{"acpi_uid",         required_argument, 0, 'U'},
 			{"verbose",          optional_argument, 0, 'v'},
 			{"version",                no_argument, 0, 'V'},
 			{"write-signature",        no_argument, 0, 'w'},
@@ -1049,7 +1064,7 @@ parse_opts(int argc, char **argv)
 		};
 
 		c = getopt_long (argc, argv,
-				 "AaBb:cDd:e:E:gH:i:l:L:n:No:Op:qt:TuU:v::Vw"
+				 "AaBb:cCDd:e:E:gH:i:l:L:n:No:Op:qt:TuU:v::Vw"
 				 "@:h",
 				 long_options, &option_index);
 		if (c == -1)
@@ -1092,6 +1107,10 @@ parse_opts(int argc, char **argv)
 		case 'c':
 			opts.create = 1;
 			break;
+		case 'C':
+			opts.create = 1;
+			opts.no_boot_order = 1;
+			break;
 		case 'D':
 			opts.deduplicate = 1;
 			break;
@@ -1102,7 +1121,13 @@ parse_opts(int argc, char **argv)
 			rc = sscanf(optarg, "%u", &num);
 			if (rc == 1) opts.edd_version = num;
 			else {
-				fprintf (stderr,"invalid numeric value %s\n",optarg);
+				fprintf (stderr,"invalid numeric value %s\n",
+					 optarg);
+				exit(1);
+			}
+			if (num != 0 && num != 1 && num != 3) {
+				fprintf (stderr, "invalid EDD version %d\n",
+					 num);
 				exit(1);
 			}
 			break;
@@ -1110,7 +1135,8 @@ parse_opts(int argc, char **argv)
 			rc = sscanf(optarg, "%x", &num);
 			if (rc == 1) opts.edd10_devicenum = num;
 			else {
-				fprintf (stderr,"invalid hex value %s\n",optarg);
+				fprintf (stderr,"invalid hex value %s\n",
+					 optarg);
 				exit(1);
 			}
 			break;
@@ -1123,28 +1149,10 @@ parse_opts(int argc, char **argv)
 			exit(0);
 			break;
 
-		case 'H': {
-			char *endptr = NULL;
-			unsigned long result;
-			result = strtoul(optarg, &endptr, 16);
-			if ((result == ULONG_MAX && errno == ERANGE) ||
-					(endptr && *endptr != '\0')) {
-				print_error_arrow("Invalid ACPI_HID value",
-					optarg,
-					(intptr_t)endptr - (intptr_t)optarg);
-				exit(1);
-			}
-			if (result > 0xffff) {
-				fprintf(stderr, "Invalid ACPI_HID value: %lX\n",
-					result);
-				exit(1);
-			}
-
-			opts.acpi_hid = result;
-			break;
-		}
 		case 'i':
 			opts.iface = optarg;
+			opts.ip_version = EFIBOOTMGR_IPV4;
+			opts.ip_addr_origin = EFIBOOTMGR_IPV4_ORIGIN_DHCP;
 			break;
 		case 'k':
 			opts.keep_old_entries = 1;
@@ -1153,7 +1161,7 @@ parse_opts(int argc, char **argv)
 			opts.loader = optarg;
 			break;
 		case 'L':
-			opts.label = optarg;
+			opts.label = (unsigned char *)optarg;
 			break;
 		case 'N':
 			opts.delete_bootnext = 1;
@@ -1211,27 +1219,6 @@ parse_opts(int argc, char **argv)
 		case 'u':
 			opts.unicode = 1;
 			break;
-
-		case 'U': {
-			char *endptr = NULL;
-			unsigned long result;
-			result = strtoul(optarg, &endptr, 16);
-			if ((result == ULONG_MAX && errno == ERANGE) ||
-					(endptr && *endptr != '\0')) {
-				print_error_arrow("Invalid ACPI_UID value",
-					optarg,
-					(intptr_t)endptr - (intptr_t)optarg);
-				exit(1);
-			}
-			if (result > 0xffff) {
-				fprintf(stderr, "Invalid ACPI_UID value: %lX\n",
-					result);
-				exit(1);
-			}
-
-			opts.acpi_uid = result;
-			break;
-		}
 		case 'v':
 			opts.verbose = 1;
 			if (optarg) {
@@ -1271,10 +1258,11 @@ int
 main(int argc, char **argv)
 {
 	char **boot_names = NULL;
-	efi_variable_t *new_boot = NULL;
+	var_entry_t *new_boot = NULL;
 	int num;
 	int ret = 0;
 
+	putenv("LIBEFIBOOT_REPORT_GPT_ERRORS=1");
 	set_default_opts();
 	parse_opts(argc, argv);
 	if (opts.showversion) {
@@ -1282,12 +1270,6 @@ main(int argc, char **argv)
 		return 0;
 	}
 
-	if (opts.iface && (
-			opts.acpi_hid < 0 || opts.acpi_uid < 0 ||
-			opts.acpi_hid > UINT32_MAX ||
-			opts.acpi_uid > UINT32_MAX))
-		errx(1, "You must specify the ACPI HID and UID when using -i.");
-
 	if (!efi_variables_supported())
 		errx(2, "EFI variables are not supported on this system.");
 
@@ -1324,7 +1306,7 @@ main(int argc, char **argv)
 			err(5, "Could not prepare boot variable");
 
 		/* Put this boot var in the right BootOrder */
-		if (new_boot)
+		if (new_boot && !opts.no_boot_order)
 			ret=add_to_boot_order(new_boot->num);
 		if (ret < 0)
 			err(6, "Could not add entry to BootOrder");
